program Project2;
{$APPTYPE CONSOLE}
uses
  sysutils,
  Classes,
  TypInfo,
  Forms,
  PAXCOMP_CONSTANTS,
  PaxCompiler,
  PaxCompilerExplorer,
  PaxCompilerDebugger,
  PaxBasicLanguage,
  PaxProgram,
  PaxRegister,
  IMPORT_Common;

type
  TMyHandler = class
  private
    L: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure SaveToDisk;
    function SavePCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
    function LoadPCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
    procedure UnknownDirectiveHandler(Sender: TPaxCompiler;
      const Directive: string; var ok: Boolean);

    function LoadPCUProgramHandler(Sender: TPaxProgram; const UnitName: String): TStream;

    procedure CreateObjectHandler(Sender: TPaxProgram;
                                  Instance: TObject);
    procedure AfterObjectCreationHandler(Sender: TPaxProgram;
                                  Instance: TObject);
    procedure DestroyObjectHandler(Sender: TPaxProgram;
                                  Instance: TObject);
    procedure AfterObjectDestructionHandler(Sender: TPaxProgram;
                                  Instance: TObject);
  end;

constructor TMyHandler.Create;
begin
  inherited;
  L := TStringList.Create;
end;

destructor TMyHandler.Destroy;
var
  I: Integer;
begin
  for I := 0 to L.Count - 1 do
    L.Objects[I].Free;

  L.Free;
  inherited;
end;

procedure TMyHandler.SaveToDisk;
var
  I: Integer;
  Stream: TMemoryStream;
  S: String;
begin
  for I := 0 to L.Count - 1 do
  begin
    S := L[I];
    Stream := TMemoryStream(L.Objects[I]);
    Stream.Position := 0;
    Stream.SaveToFile(S + '.PCU');
  end;
end;

function TMyHandler.SavePCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
begin
  result := TMemoryStream.Create;
  L.AddObject(UnitName, result);
end;

function TMyHandler.LoadPCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
var
  I: Integer;
begin
  result := nil;
  for I := 0 to L.Count - 1 do
    if CompareText(L[I], UnitName) = 0 then
    begin
      result := TStream(L.Objects[I]);
      result.Position := 0;
      Exit;
    end;
end;

function TMyHandler.LoadPCUProgramHandler(Sender: TPaxProgram; const UnitName: String): TStream;
var
  I: Integer;
begin
  result := nil;
  for I := 0 to L.Count - 1 do
    if CompareText(L[I], UnitName) = 0 then
    begin
      result := TStream(L.Objects[I]);
      result.Position := 0;
      Exit;
    end;
end;

procedure TMyHandler.CreateObjectHandler(Sender: TPaxProgram;
                                  Instance: TObject);
var
  pti: PTypeInfo;
  ptd: PTypeData;
  S: String;
begin
  writeln('OnCreateObject:');

  if Instance is TForm then
  begin
    pti := Instance.ClassInfo;
    ptd := GetTypeData(pti);
    if FileExists(ptd^.UnitName + '.dfm') then
      Sender.LoadDFMFile(Instance, ptd^.UnitName + '.dfm');
  end;
end;

procedure TMyHandler.AfterObjectCreationHandler(Sender: TPaxProgram;
                                  Instance: TObject);
begin
  writeln('OnAfterObjectCreation:');
end;

procedure TMyHandler.DestroyObjectHandler(Sender: TPaxProgram;
                                  Instance: TObject);
begin
  writeln('OnDestroyObject:');
end;

procedure TMyHandler.AfterObjectDestructionHandler(Sender: TPaxProgram;
                                  Instance: TObject);
begin
  writeln('OnAfterObjectDestruction:');
end;

procedure TMyHandler.UnknownDirectiveHandler(Sender: TPaxCompiler;
  const Directive: string; var ok: Boolean);
begin
  ok := true;
end;

var
  MyHandler: TMyHandler;

var
  PaxCompiler1: TPaxCompiler;
  PaxProgram1: TPaxProgram;
  PaxPascalLanguage1: TPaxPascalLanguage;
begin
  // Build all units, save pcu-files and create compiled script

  MyHandler := TMyHandler.Create;
  try

    PaxCompiler1 := TPaxCompiler.Create(nil);
    PaxCompiler1.OnUnknownDirective := MyHandler.UnknownDirectiveHandler;
//    PaxCompiler1.OnSavePCU := MyHandler.SavePCUCompilerHandler;
//    PaxCompiler1.OnLoadPCU := MyHandler.LoadPCUCompilerHandler;

    PaxProgram1 := TPaxProgram.Create(nil);
    PaxProgram1.OnCreateObject := MyHandler.CreateObjectHandler;
    PaxProgram1.OnAfterObjectCreation := MyHandler.AfterObjectCreationHandler;
    PaxProgram1.OnDestroyObject := MyHandler.DestroyObjectHandler;

    PaxPascalLanguage1 := TPaxPascalLanguage.Create(nil);
    try
      PaxCompiler1.RegisterLanguage(PaxPascalLanguage1);
      PaxCompiler1.AddModule('1', 'Pascal');
//      PaxCompiler1.AddCodeFromFile('1', 'script.txt');
      PaxCompiler1.AddCodeFromFile('1', 'script6.txt');

      if PaxCompiler1.Compile(PaxProgram1, true, false) then
      // build with run-time packages
      begin
        PaxProgram1.SaveToFile('script.bin');
//        PaxProgram1.Run;
//        writeln('Press any key...');
//        Readln;
//        Exit;
      end
      else
      begin
        writeln(PaxCompiler1.ErrorMessage[0]);
        writeln(PaxCompiler1.ErrorLine[0]);
        writeln('Press any key...');
        Readln;
        Exit;
      end;
    finally
      PaxCompiler1.Free;
      PaxProgram1.Free;
      PaxPascalLanguage1.Free;
    end;

//    MyHandler.SaveToDisk;

    // Use compiled units (pcu-files) at run-tme as run-time packages

    PaxProgram1 := TPaxProgram.Create(nil);
    PaxProgram1.OnCreateObject := MyHandler.CreateObjectHandler;
    PaxProgram1.OnAfterObjectCreation := MyHandler.AfterObjectCreationHandler;
    PaxProgram1.OnDestroyObject := MyHandler.DestroyObjectHandler;
    try
//    PaxProgram1.OnLoadPCU := MyHandler.LoadPCUProgramHandler;

      PaxProgram1.LoadFromFile('script.bin');
      PaxProgram1.Run;
    finally
      PaxProgram1.Free;
    end;

  finally
    MyHandler.Free;
  end;

  writeln('Press any key...');
  Readln;
end.

